// Copyright (c) 2017, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library editex.demo;

import 'package:eqlib/eqlib.dart';
import 'package:eqlib/latex.dart';
import 'package:editex/editex.dart';

final _defaultCommands = {
  'a': r'{}_\text{?}a',
  'a1': r'a_1',
  'a2': r'a_2',
  'sqrt': r'\sqrt{${0}\,}',
  'root': r'\sqrt[${0}]{${1}\,}',
  'log': r'\log_{${0}}${:1(+).}',
  'sin': r'\sin${:0(+).}',
  'cos': r'\cos${:0(+).}',
  'celcius': r'${.0(*)}^\circ',
  'lim': r'\lim_{${0}\to${1}}${2(+).}',
  'diff': r'\frac{\partial}{\partial${:0(+)}}${:1(+).}',
  'frac': r'\frac{\,${0}\,}{\,${1}\,}',
  'int': r'\displaystyle\int_{${0}}^{${1}}${2}~\text{d}${:3(+).}',
};

/// Test implementation of [EdiTeXInterface].
class EdiTeXTestInterface implements EdiTeXInterface {
  final ctx = new SimpleExprContext();
  final commands = new Map<String, EdiTeXCommand>();
  final instantCommands = new Map<String, EdiTeXCommand>();

  EdiTeXTestInterface() {
    // Initialize commands.
    _defaultCommands.forEach((key, templateStr) {
      final t = parseLaTeXTemplate(templateStr, ctx.operators);
      final args = new List<String>.generate(t.parameterCount, (i) => '\$$i');
      commands[key] = new EdiTeXCommand(key, t, '$key(${args.join(',')})');
    });

    // Initialize instant commands.
    final parse = (String str) => parseLaTeXTemplate(str, ctx.operators);
    instantCommands['('] =
        new EdiTeXCommand('(', parse(r'\left(${0}\right)'), r'($0)');
    instantCommands['['] =
        new EdiTeXCommand('(', parse(r'\left[${0}\right]'), r'brackets($0)');
    instantCommands['^'] = new EdiTeXCommand('^', parse(r'^{${0}}'), r'^($0)');
    instantCommands['_'] = new EdiTeXCommand('_', parse(r'_{${0}}'), r'_($0)');
    instantCommands['*'] = new EdiTeXCommand('*', parse(r'{}\cdot{}'), '*');
    instantCommands['='] = new EdiTeXCommand('=', parse(r'{}={}'), '=');
    instantCommands['+'] = new EdiTeXCommand('+', parse(r'{}+{}'), '+');
    instantCommands['-'] = new EdiTeXCommand('-', parse(r'{}-{}'), '-');
    instantCommands['~'] = new EdiTeXCommand('~', parse(r'-'), '-');
    instantCommands['!'] = new EdiTeXCommand('!', parse(r'!'), '!');
  }

  @override
  bool hasCommand(String command) => commands.containsKey(command);

  @override
  bool hasInstantCommand(String command) =>
      instantCommands.containsKey(command);

  @override
  EdiTeXCommand resolveCommand(command) => commands[command];

  @override
  EdiTeXCommand resolveInstantCommand(command) => instantCommands[command];
}
